# -*- coding: ISO-8859-1 -*-

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses
import oss

import config
import events
import statusbar
import window


class mixerwin(window.window):

    def __init__(self, screen, maxh, maxw, colors,
                 channel, timer, device_channel, keybindings):
        self.channel = channel
        self.timer = timer
        self.mixer_device = device_channel[0]
        self.mixer_channel = eval("oss.%s" % device_channel[1])
        self.mixer = oss.open_mixer(self.mixer_device)
        self.keybindings = keybindings
        self.level = self.mixer.read_channel(self.mixer_channel)

        # for identification purposes, we only generate this once
        self.hidewindowevent = events.hidewindow(self)
        
        self.hide()

        self.channel.subscribe(events.keypressed, self.keypressed)
        self.channel.subscribe(events.mouseevent, self.mouseevent)
        self.channel.subscribe(events.hidewindow, self.hidewindow)
        self.channel.subscribe(events.focuschanged, self.focuschanged)

    def changevolume(self, change):
        oldlevel = self.mixer.read_channel(self.mixer_channel)
        self.level= ( max(0, min(oldlevel[0]+change, 100)),
                      max(0, min(oldlevel[1]+change, 100)) )
        self.mixer.write_channel(self.mixer_channel, self.level)
        
    # event handler

    def keypressed(self, event):
        key = event.key
        if key in self.keybindings["volumeup"]:
            self.changevolume(+5)
        elif key in self.keybindings["volumedown"]:
            self.changevolume(-5)
        else:
            if self.hasfocus():
                self.hide()
                raise events.TerminateEventProcessing
            return

        self.update()
        if config.mixerwinautoclosetime:
            self.timer.sendeventin(self.hidewindowevent,
                                   config.mixerwinautoclosetime,
                                   replace=1)
        
        raise events.TerminateEventProcessing

    def mouseevent(self, event):
        if self.hasfocus():
            self.hide()
            raise events.TerminateEventProcessing

    def focuschanged(self, event):
        # we either have focus, or we disappear...
        if not self.hasfocus():
            self.hide()
        else:
            sbar = statusbar.generatedescription("global", "volumedown")
            sbar += statusbar.separator
            sbar += statusbar.generatedescription("global", "volumeup")

            events.hub.notify(events.updatestatusbar(sbar))

    def update(self):
        self.top()
        window.window.update(self)
        self.win.addstr(self.iy, self.ix, _("Volume:"), self.colors["description"])
        self.win.addstr(" %3d " % self.level[0], self.colors["content"])
 
        percent = self.barlen*self.level[0]/100
        self.win.addstr("#"*percent, self.colors["barhigh"])
        self.win.addstr("#"*(self.barlen-percent), self.colors["bar"])


class popupmixerwin(mixerwin):

    """ mixer which appears as a popup at the center of the screen """
    
    def __init__(self, screen, maxh, maxw, colors,
                 channel, timer, device_channel, keybindings):
        # calculate size and position
        self.barlen = 20
        h = 3
        w = len(_("Volume:")) + 7 + self.barlen
        y = (maxh-h)/2
        x = (maxw-w)/2

        window.window.__init__(self,
                               screen, h, w, y, x,
                               colors,
                               _("Mixer"), 0)
                
        mixerwin.__init__(self,
                          screen,
                          maxh, maxw, colors,
                          channel, timer, device_channel, keybindings)

    def resize(self, maxh, maxw):
        h = 3
        w = len(_("Volume:")) + 7 + self.barlen
        y = (maxh-h)/2
        x = (maxw-w)/2
        window.window.resize(self, h, w, y, x)
        
        
class statusbarmixerwin(mixerwin):

    """ mixer which appears in the statusbar """
    
    def __init__(self, screen, maxh, maxw, colors,
                 channel, timer, device_channel, keybindings):
        # calculate size and position
        self.barlen = 20

        window.window.__init__(self,
                               screen, 1, maxw, maxh-1, 0, colors)
                
        mixerwin.__init__(self,
                          screen,
                          maxh, maxw, colors,
                          channel, timer, device_channel, keybindings)

    def resize(self, maxh, maxw):
        window.window.resize(self, 1, maxw, maxh-1, 0)
